from rocolib.api.composables.Composable import Composable, output, widget
import networkx as nx
from circlify import circlify
from rocolib.utils.nx2go import GraphVisualization as gv
from dash import dcc


class ComponentComposable(Composable):
    def __init__(self, component):
        self.c = component
        self.net = nx.DiGraph()
        self.net.graph['depth'] = 0

    def append(self, c2, name, **kwargs):
        sc = c2.c
        scname = sc.name
        scclass = sc.cName
        scnet = c2.net
        
        self.net.add_node(scname, name=name, component=scclass, net=scnet)
        self.net.graph['depth'] = max(self.net.graph['depth'], scnet.graph['depth']+1)

    def attach(self, fromPort, toPort, **kwargs):
        self.net.add_edge(fromPort.parent.name, toPort.parent.name)

    def _makeComponentMap(self, scd=None, full=None, idnum = 0):
        if scd is None:
            g = self.net
            full = nx.DiGraph()
            full.add_node(0, name="", component=self.c.name)
            title = f"<b>{self.c.name}</b>"
        else:
            try:
                g = scd['net']
            except KeyError:
                print(scd, idnum)
                raise
            title = f"<b>{scd['component']}</b><br>{scd['name']}"

        data = dict(id=idnum,
                    datum=(g.graph['depth']+1)**2,
                    children=[dict(id=len(full), datum=0.5)])
        full.add_node(len(full), title=title)

        if g:
            f = nx.convert_node_labels_to_integers(g, len(full))
            full = nx.disjoint_union(full, f)
            for sc, scd in f.nodes(data=True):
                full, cd = self._makeComponentMap(scd, full, sc)
                data["children"].append(cd)
        return full, data

    def _drawComponentTree(self):
        g, data = self._makeComponentMap()
        circles = circlify([data])

        tree = gv(g,
            pos = {c.ex['id']: (c.x, c.y) for c in circles},
            node_size = {c.ex['id']: c.r for c in circles},
            node_text = lambda k, v : v.get("title", ""),
            node_border_width = lambda k, v: "title" not in v and 2 or 0,
            edge_width = 0,
        )
        fig = tree.create_figure(width=1024, height=768)
        return fig

    @output("map.png", "component topology map", binary=True)
    def drawComponentMap(self, fp, **ka):
        fig = self._drawComponentTree()
        fig.write_image(fp, format="png")

    @widget('displaymap', "component topology map")
    def displayMap(self, fp, **ka):
        return dcc.Graph(figure = self._drawComponentTree())
