import logging
from io import BytesIO, StringIO


log = logging.getLogger(__name__)

def output(stub, desc, binary=False, widget=False, **kwargs):
    def wrap(f):
        def inner(self, fp, **ka):
            ka.update(kwargs)
            return f(self, fp, **ka)
        inner.output = dict(f=inner, desc=desc, stub=stub, binary=binary, widget=widget)
        return inner
    return wrap

def widget(stub, desc, **kwargs):
    return output(stub, desc, widget=True, **kwargs)

class Composable:
    def new(self):
        return self.__class__()
    def append(self, newComposable, newPrefix, **kwargs):
        pass
    def attach(self, fromPort, toPort, **kwargs):
        pass

    @classmethod
    def listOutputs(cls, widgets=False):
        outputs = {}
        for methodname in dir(cls):
            method = getattr(cls, methodname)
            if hasattr(method, 'output'):
                out = method.output
                keyword = out['stub'].split('.')[0]
                if out['widget'] == widgets or widgets is None:
                    outputs[keyword] = out
        return outputs

    def makeOutput(self, name, outputs=(), filedir=None, widgets=False, **ka):
        basename = None
        if filedir:
            basename = filedir + f"/{name}-"

        knownouts = self.listOutputs(widgets)

        rets = {}
        if outputs is True:
            outputs = knownouts

        for k in outputs:
            if k in knownouts:
                v = knownouts[k]
                log.info("Generating %s ..." % v['desc'])
                if basename and v['stub']:
                    fn = basename+v['stub']
                    fp = open(fn, v['binary'] and 'wb' or 'w')
                    rets[k] = fn
                else:
                    fp = v['binary'] and BytesIO() or StringIO()
                ret = v['f'](self, fp, **ka)
                if widgets:
                    rets[k] = dict(desc=v['desc'], widget=ret)
                if k not in rets:
                    rets[k] = fp.getvalue()
                fp.close()
        return rets
