from rocolib.library import getComponent
from rocolib.api.components.Component import newComponent
from rocolib.library import save
from rocolib.api.Function import Function

c = newComponent("ESPSeg")

c.addParameter("length", 90, paramType="length")
c.addParameter("width", 60, paramType="length")
c.addParameter("height", 40, paramType="length")
c.addParameter("tire_thickness", 0, paramType="length")


c.addParameter("driveservo", "fs90r", values=getComponent("ServoMotor").dims)

c.addSubcomponent("brain", "MountedBrains", inherit="brain", prefix=None)
c.addSubcomponent("right", "Wheel", invert=True, inherit="angle")
c.addSubcomponent("left", "Wheel", invert=True, inherit="angle")

c.addConstraint(("right", "tire_thickness"), "tire_thickness")
c.addConstraint(("left", "tire_thickness"), "tire_thickness")

# Constant thickness of main body
def depthfn(params = None, fnmod = None):
    if params is None: params = []
    if fnmod is None: fnmod = "%s"
    return ["brain", "driveservo"] + params, \
            fnmod % "max(x[0].get('height'), x[1].get('motorwidth'))"

# Set microcontroller
c.addConstraint(("brain", "depth"), *depthfn())
c.addConstraint(("brain", "length"), "width")

for servo in ("right", "left"):
    c.addConstraint((servo, "length"),
                    ("length", "brain"),
                    "x[0] - x[1].get('width')")
    c.addConstConstraint((servo, "center"), False)
    c.addConstraint((servo, "servo"), "driveservo")
    c.addConstraint((servo, "radius"), "height")

c.addConstConstraint(("right","flip"), True)

# connections
c.addConnection(("brain", "topedge0"),
                ("right", "topedge0"),
                angle=-90)
c.addConnection(("brain", "botedge0"),
                ("left", "botedge0"),
                angle=-90)

# Sheath
c.addParameter("battery", 7, paramType="length")

c.addSubcomponent("sheath", "SimpleRectBeam")
# c.addConstConstraint(("sheath","phase"), 1)
c.addConstraint(("sheath","length"), "length")
c.addConstraint(("sheath","depth"), "width")
c.addConstraint(("sheath","width"), *depthfn(["battery"], "%s + x[2]"))

c.addSubcomponent("sheathsplit", "SplitEdge")
c.addConstraint(("sheathsplit","toplength"), "width", "(x,)")
c.addConstraint(("sheathsplit","botlength"), ("driveservo", "width"),
        "(x[0].get('motorheight'), \
          x[1] - 2*x[0].get('motorheight'), \
          x[0].get('motorheight'))")

c.addConnection(("left", "topedge1"),
                ("sheathsplit", "botedge2"),
                angle=180)

'''
c.addConnection(("right", "botedge1"),
                ("sheathsplit", "botedge0"),
                angle=180, tabWidth=40)
'''

c.addConnection(("sheathsplit", "topedge0"),
                ("sheath", "botedge1"))

# Tail
c.addSubcomponent("tail", "Tail", inherit=("flapwidth", "tailwidth"), prefix=None)
c.addConstraint(("tail","width"), "width")
c.addConstraint(("tail","height"), *depthfn(["height"], "%s/2.+x[2]"))
c.addConstraint(("tail","depth"), *depthfn(["battery"], "%s+x[2]"))

c.addConnection(("tail", "topedge"),
                ("sheath", "topedge1"),
                angle=90)

c.addSubcomponent("tailsplit", "SplitEdge")
c.addConstraint(("tailsplit","toplength"), "width", "(x,)")
c.addConstraint(("tailsplit","botlength"), ("width", "flapwidth"), "(x[0]*(1-x[1])/2., x[0]*x[1], x[0]*(1-x[1])/2.)")

c.addConnection(("sheath", "topedge3"),
                ("tailsplit", "topedge0"))

c.addConnection(("tail", "flapedge"),
                ("tailsplit", "botedge1"),
                angle=90, tabWidth=Function(*depthfn(["battery"], "%s+x[2]")))

# USB charging port for battery
c.addSubcomponent("usb", "Cutout")
c.addConstConstraint(("usb", "dy"), 9)
c.addConstConstraint(("usb", "dx"), 4)

c.addConnection(("sheath", "face1"),
                ("usb", "decoration"),
                mode="hole",
                offset=Function(*depthfn(["length", "battery"], "(4-(%s+x[3])/2, 0.5 * x[2] - 15)")))

c.addConnection(("right", "face0"),
                ("sheath", "face2"), copyDecorations=True, transform=False)
c.addConnection(("left", "face0"),
                ("sheath", "face0"), copyDecorations=True, transform=False)
c.addConnection(("brain", "face1"),
                ("sheath", "face1"), copyDecorations=True, transform=False)

save(c, True)
