from rocolib.library import getComponent
from rocolib.api.components.Component import newComponent
from rocolib.library import save
from rocolib.api.Function import Function

c = newComponent("ServoMount")

c.addParameter("servo", "fs90r", values=getComponent("ServoMotor").dims)
c.addParameter("flip", False, valueType="bool")
c.addParameter("center", True, valueType="bool")
c.addParameter("shift", 0, paramType="length")

# XXX TODO: Define type: tuple of two numbers
c.addParameter("offset", 0)

c.addSubcomponent("beam", "SimpleRectBeam", inherit=("length", "addTabs"), prefix=None)
c.addSubcomponent("mount", "Cutout")

c.addConstraint(("mount", "dx"), "servo", 'x.get("motorwidth") * 0.99')
c.addConstraint(("mount", "dy"), "servo", 'x.get("motorlength")')

c.addConstraint(("beam", "width"), "servo", 'x.get("motorwidth")')
c.addConstraint(("beam", "depth"), "servo", 'x.get("motorheight")')
c.addConstraint(("beam", "length#minValue"), "servo", 'x.get("motorlength") + 2 * x.get("shoulderlength")')

c.inheritAllInterfaces("beam", prefix=None)
c.inheritAllInterfaces("mount")
c.addConnection(("beam", "face0"),
                ("mount", "decoration"),
                mode="hole", offset=Function(params="offset"))

save(c, True)
