from rocolib.library import getComponent
from rocolib.api.composables.graph.Face import Face


ThruHole = getComponent("ThruHole").__class__

class Brains(ThruHole):

    brains = dict(
        proMini = dict(
            length    = 39,
            width     = 19,
            height    = 9,
            nrows     = 12,
            ncols     = 2,
            rowsep    = 0.1 * 25.4,
            colsep    = 0.6 * 25.4,
            origin    = (0,0),
            extras    = (),
        ),
        nodeMCU = dict(
            length    = 59.5,
            width     = 44,
            height    = 13,
            nrows     = 15,
            ncols     = 2,
            rowsep    = 0.1 * 25.4,
            colsep    = 0.9 * 25.4,
            origin    = (7.5, -4.5),
            extras    = dict(
                servoPins = dict(center = (-17.25, -14), size = (8, 27)),
            ),
        ),
    )

    def define(self):
        ThruHole.define(self)
        self.addParameter("brain", "nodeMCU", values=self.brains, overrides = dict(
            nrows = 'c.p.brain.get("nrows")',
            ncols = 'c.p.brain.get("ncols")',
            rowsep = 'c.p.brain.get("rowsep")',
            colsep = 'c.p.brain.get("colsep")',
        ))

    def assemble(self):
        ThruHole.assemble(self)
        brain = self.p.brain
        self.graph.dotransform(origin = brain.get("origin"))
        for n, e in brain.get("extras").items():
            x, y = e.get("center", (0,0))
            dx, dy = e.get("size", (1,1))
            self.addFace(Face(n,
                          ((x-dx/2, y-dy/2), (x+dx/2, y-dy/2), (x+dx/2, y+dy/2), (x-dx/2, y+dy/2)),
                          recenter=False
            ))

if __name__ == "__main__":
    Brains.test()
