from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Face, Rectangle
import rocolib.utils.numsym as np

class RectBeam(FoldedComponent):
  def define(self):
    self.addParameter("length", 100, paramType="length")
    self.addParameter("width", 20, paramType="length")
    self.addParameter("depth", 50, paramType="length")

    self.addParameter("phase", 0, valueType="int")

    self.addParameter("angle", overrides=("tangle", "bangle"))
    self.addParameter("tangle", 80, paramType="angle", maxValue=180)
    self.addParameter("bangle", 135, paramType="angle", maxValue=180)

    self.addParameter("root", 0, values=[0,1,2,3])
    self.addParameter("addTabs", True, valueType="bool")

    for i in range(4):
      self.addEdgeInterface("topedge%d" % i, "r%d.e0" % i, ["width", "depth"][i % 2])
      self.addEdgeInterface("botedge%d" % i, "r%d.e2" % i, ["width", "depth"][i % 2])
      self.addFaceInterface("face%d" % i, "r%d" % i)
    self.addEdgeInterface("tabedge", "r3.e1", "length")
    self.addEdgeInterface("slotedge", "r0.e3", "length")

  def assemble(self):
    bangle = 90 - self.p.bangle
    tangle = 90 - self.p.tangle

    root = self.p.root

    length = self.p.length
    width = self.p.width
    depth = self.p.depth
    phase = self.p.phase

    def dl(a):
        return np.tan(np.deg2rad(a)) * depth

    rs = []
    rs.append(Rectangle("", width, length))
    rs.append(Face("", (
      (depth, dl(tangle)),
      (depth, length - dl(bangle)),
      (0, length), 
      (0,0)
    )))
    rs.append(Rectangle("", width, length - dl(tangle) - dl(bangle)))
    rs.append(Face("", (
      (0, length), 
      (0,0),
      (depth, dl(bangle)),
      (depth, length - dl(tangle))
    )))

    for i in range(phase):
      rs.append(rs.pop(0))

    fromEdge = None
    for i in range(4):
      self.attachEdge(fromEdge, rs[i], "e3", prefix="r%d"%i, angle=90, root=((i == root)))
      fromEdge = 'r%d.e1' % i
      self.setFaceInterface("face%d" % i, "r%d" % ((i-phase)%4))

    slotEdge = "r0.e3"
    tabEdge = "r3.e1"

    if self.p.addTabs:
        self.addTab(slotEdge, tabEdge, angle= 90, width=min(10, [depth, width][phase % 2]))

if __name__ == "__main__":
  RectBeam.test()
