from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Face, Rectangle, RightTriangle


class Tail(FoldedComponent):
    def define(self):
        self.addParameter("height", 60, paramType="length")
        self.addParameter("depth", 20, paramType="length")
        self.addParameter("width", 80, paramType="length")
        self.addParameter("flapwidth", 0.2, paramType="ratio")
        self.addParameter("tailwidth", 1/3, paramType="ratio")

        self.addEdgeInterface("topedge", "spacer.e4", "width")
        self.addEdgeInterface("flapedge", "tail.e2", "width")


    def assemble(self):
        h = self.p.height
        w = self.p.width
        d = self.p.depth

        flapwidth = self.p.flapwidth
        tailwidth = self.p.tailwidth
        lr0 = (1-flapwidth)/2.
        lr1 = 1-lr0
        lt0 = (1-tailwidth)/2.
        lt1 = 1-lt0

        s = Face("", ((w*lr0, 0), (w*lr1, 0), (w, 0), (w, 0.1), (0, 0.1), (0, 0)))
        t = Face("", ((w*lr0, 0), (w*lr0, d), (w*lr1, d), (w*lr1, 0), (w, 0), (w*lt1, h), (w*lt0, h), (0, 0)))
        f1 = RightTriangle("", h, w*lt0)
        f2 = RightTriangle("", w*lt0, h)

        self.addFace(s, "spacer")
        self.attachEdge("spacer.e2", t, "e0", "tail", angle=0)
        self.mergeEdge("spacer.e0", "tail.e4", angle=0)
        self.attachEdge("tail.e5", f2, "e1", "f1", angle=-135)
        self.attachEdge("tail.e7", f1, "e1", "f2", angle=-135)

if __name__ == "__main__":
    Tail.test()

