from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Face


class Trapezoid(FoldedComponent):

  def define(self):
    self.addParameter("lt", 100, paramType="length")
    self.addParameter("l", 300, paramType="length")
    self.addParameter("w", 400, paramType="length")
    self.addParameter("center", True, valueType="bool")

    self.addEdgeInterface("b", "e0", "l")
    self.addEdgeInterface("r", "e1", None)
    self.addEdgeInterface("t", "e2", "lt")
    self.addEdgeInterface("l", "e3", None)
    self.addFaceInterface("face", "f")

  def assemble(self):
    x1 = self.p.l/2
    x2 = self.p.lt/2
    dy = self.p.w/2

    if self.p.center:
        xa, xb = x2, -x2
    else:
        xa, xb = x1, x1 - 2*x2

    self.addFace(Face("f", ((x1, -dy), (xa, dy), (xb, dy), (-x1, -dy)), recenter=False))

if __name__ == "__main__":
    import sympy

    Trapezoid.test()

    # Test sympy
    r = Trapezoid()
    r.makeOutput(useDefaultParameters=False, default=False)
    g = r.getGraph()
    for f in g.faces:
        sympy.pprint(f.transform3D)
        sympy.pprint(f.get3DCoords())

